use num::*;

pub mod strings;

pub fn matrix(zero: bool, diag: bool, non_diag: bool) -> Vec<(i32, i32)> {
    const MIN: i32 = -1;
    const MAX: i32 = 1;
    let mut d = Vec::new();
    for i in MIN..=MAX {
        for j in MIN..=MAX {
            let ia = i.abs();
            let ja = j.abs();
            if zero && i == 0 && j == 0 {
                d.push((i, j));
            }
            if diag && ia == 1 && ja == 1 {
                d.push((i, j));
            }
            if non_diag && ia != ja && ia + ja == 1 {
                d.push((i, j));
            }
        }
    }
    d
}

#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
struct Kartesian<T>
where
    T: Integer,
{
    x: T,
    y: T,
}
#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
struct Euclidian<T>
where
    T: Integer,
{
    x: T,
    y: T,
    z: T,
}

type Table = Vec<Vec<char>>;

#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
enum KartesianDirection {
    TopLeft,
    Top,
    TopRight,
    Left,
    None,
    Right,
    BottomLeft,
    Bottom,
    BottomRight,
}

impl KartesianDirection {
    pub fn vector<T: Signed + Integer + Zero + From<i8>>(self) -> Kartesian<T> {
        const BELOW: i8 = -1;
        const ON: i8 = 0;
        const UPPER: i8 = 1;
        Kartesian::<T> {
            x: match self {
                Self::TopLeft | Self::Top | Self::TopRight => BELOW.into(),
                Self::Left | Self::None | Self::Right => ON.into(),
                Self::BottomLeft | Self::Bottom | Self::BottomRight => UPPER.into(),
            },
            y: match self {
                Self::TopLeft | Self::Left | Self::BottomLeft => BELOW.into(),
                Self::Top | Self::None | Self::Bottom => ON.into(),
                Self::TopRight | Self::Right | Self::BottomRight => UPPER.into(),
            },
        }
    }
}
